package video.pano.pano_rtc

import android.content.Context
import android.view.View
import com.pano.rtc.api.RtcView
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import video.pano.RendererCommon.RendererEvents
import video.pano.pano_rtc.api.IRtcSurfaceView

class PanoNativeViewFactory : PlatformViewFactory(StandardMessageCodec.INSTANCE) {

    override fun create(context: Context, viewId: Int, args: Any?): PlatformView {
        return PanoNativeView(context, viewId)
    }
}

class PanoNativeView(context: Context, viewId: Int) : FlutterWrapper(), PlatformView,
        IRtcSurfaceView, MethodChannel.MethodCallHandler {
    private val view = RtcView(context)

    init {
        methodChannel = PanoRtcPlugin.createMethodChannel("pano_rtc/view_surface_$viewId")
        methodChannel.setMethodCallHandler(this)
        view.init(object : RendererEvents {
            override fun onFirstFrameRendered() {}
            override fun onFrameResolutionChanged(i: Int, i1: Int, i2: Int) {}
        })
    }

    override fun getView(): View {
        return view
    }

    override fun dispose() {
        methodChannel.setMethodCallHandler(null)
    }

    override fun startVideo(params: Map<String, *>, callback: Callback) {
        view.mirror = true
        val newParams = params + Pair("view", view)
        PanoRtcPlugin.engineManager.startVideo(newParams, callback)
    }

    override fun subscribeVideo(params: Map<String, *>, callback: Callback) {
        view.mirror = true
        val newParams = params + Pair("view", view)
        PanoRtcPlugin.engineManager.subscribeVideo(newParams, callback)
    }

    override fun startPreview(params: Map<String, *>, callback: Callback) {
        PanoRtcPlugin.engineManager.startPreview(params, callback)
    }

    override fun startVideoWithStreamId(params: Map<String, *>, callback: Callback) {
        PanoRtcPlugin.engineManager.videoStreamEngine?.startVideo(params, callback)
    }

    override fun subscribeVideoWithStreamId(params: Map<String, *>, callback: Callback) {
        PanoRtcPlugin.engineManager.videoStreamEngine?.subscribeVideo(params, callback)
    }
}