package video.pano.pano_rtc

import com.pano.rtc.api.Constants
import com.pano.rtc.api.PanoAnnotationManager
import video.pano.pano_rtc.api.IRtcAnnotationManager
import video.pano.pano_rtc.events.RtcAnnotationManagerEvent

class RtcAnnotationManager(private val manager: PanoAnnotationManager?) : FlutterWrapper(),
        IRtcAnnotationManager {
    private val annoMap = HashMap<String, RtcAnnotation>()

    init {
        methodChannel = PanoRtcPlugin.createMethodChannel("pano_rtc/api_annotationMgr")
        eventChannel = PanoRtcPlugin.createEventChannel("pano_rtc/events_annotationMgr")
        methodChannel.setMethodCallHandler(this)
        eventChannel?.setStreamHandler(this)
        manager?.setCallback(RtcAnnotationManagerEvent { methodName, data ->
            emit(methodName, data)
        })
    }

    override fun getVideoAnnotation(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Long
        val streamId = params["streamId"] as Int
        val key = "$userId-$streamId"
        var annotation = annoMap[key]
        if (annotation == null) {
            annotation = RtcAnnotation(manager?.getVideoAnnotation(userId, streamId), key)
            annoMap[key] = annotation
        }
        callback.success(Constants.QResult.OK)
    }

    override fun getShareAnnotation(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Long
        val key = userId.toString()
        var annotation = annoMap[key]
        if (annotation == null) {
            annotation = RtcAnnotation(manager?.getShareAnnotation(userId), key)
            annoMap[key] = annotation
        }
        callback.success(Constants.QResult.OK)
    }

    fun getAnnotationById(channelId: String): RtcAnnotation? {
        return annoMap[channelId]
    }

}