//
//  RtcAnnotation.swift
//  pano_rtc
//
//  Created by mark on 2020/12/30.
//

import Foundation
import PanoRtc

protocol RtcAnnotationInterface {
    func setRoleType(_ params: NSDictionary, _ callback: Callback)
    
    func startAnnotation(_ params: NSDictionary, _ callback: Callback)
    
    func stopAnnotation(_ callback: Callback)
    
    func setVisible(_ params: NSDictionary, _ callback: Callback)
    
    func setToolType(_ params: NSDictionary, _ callback: Callback)
    
    func setLineWidth(_ params: NSDictionary, _ callback: Callback)
    
    func setColor(_ params: NSDictionary, _ callback: Callback)
    
    func setFontStyle(_ params: NSDictionary, _ callback: Callback)
    
    func setFontSize(_ params: NSDictionary, _ callback: Callback)
    
    func undo(_ callback: Callback)
    
    func redo(_ callback: Callback)
    
    func clearUserContents(_ params: NSDictionary, _ callback: Callback)
    
    func clearContents(_ callback: Callback)
    
    func snapshot(_ params: NSDictionary, _ callback: Callback)
}

protocol RtcAnnotationDelegate: AnyObject {
    func emit(_ methodName: String, _ data: Dictionary<String, Any?>?)
}

@objc
class RtcAnnotation: NSObject, RtcAnnotationInterface {
    weak var delegate: RtcAnnotationDelegate?
    private var annotation: PanoRtcAnnotation?
    private lazy var annotationDelegate: PanoRtcAnnotationDelegateHandler = {
        PanoRtcAnnotationDelegateHandler() { [weak self] methodName, data in
            self?.delegate?.emit(methodName, data)
        }
    }()
    
    convenience init(annotation: PanoRtcAnnotation?) {
        self.init()
        self.annotation = annotation
        self.annotation?.setDelegate(self.annotationDelegate)
    }
    
    @objc func setRoleType(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.setRoleType(PanoWBRoleType(rawValue: params["type"] as! Int)!))
    }
    
    @objc func startAnnotation(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.start(params["view"] as! UIView))
    }
    
    @objc func stopAnnotation(_ callback: Callback) {
        callback.code(annotation?.stop())
    }
    
    @objc func setVisible(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.setVisible(params["visible"] as! Bool))
    }
    
    @objc func setToolType(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.setToolType(PanoWBToolType(rawValue: params["type"] as! Int)!))
    }
    
    @objc func setLineWidth(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.setLineWidth(params["size"] as! UInt32))
    }
    
    @objc func setColor(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.setColor(PanoWBColor(map: params["color"] as! Dictionary)))
    }
    
    @objc func setFontStyle(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.setFontStyle(PanoWBFontStyle(rawValue: params["style"] as! Int)!))
    }
    
    @objc func setFontSize(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.setFontSize(params["size"] as! UInt32))
    }
    
    @objc func undo(_ callback: Callback) {
        callback.code(annotation?.undo())
    }
    
    @objc func redo(_ callback: Callback) {
        callback.code(annotation?.redo())
    }
    
    @objc func clearUserContents(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.clearUserContents(params["userId"] as! UInt64))
    }
    
    @objc func clearContents(_ callback: Callback) {
        callback.code(annotation?.clearContents())
    }
    
    @objc func snapshot(_ params: NSDictionary, _ callback: Callback) {
        callback.code(annotation?.snapshot(params["outputDir"] as! String))
    }
}
